﻿// Constantes
const KEY_CODE_A		= 32;
const KEY_CODE_LEFT		= 37;
const KEY_CODE_UP		= 38;
const KEY_CODE_RIGHT	= 39;
const KEY_CODE_DOWN		= 40;
const FORCUS_MODE		= { MAIN: 0, OPTION: 1, CONFIRM_EXIT: 2 };
const MAIN_MENU			= ["button_continue", "button_guide_basic", "button_guide_advance", "button_guide_2p", "button_option", "button_exit", "about"];
const OPTION_MENU		= ["button_option_camera_h", "button_option_camera_v", "button_option_pump_v", "button_option_fpcamera_h", "button_option_fpcamera_v", "button_option_ok"];

// Variables Globales
var sGameName			= ""; //Variable de tipo de juego (Mario64,MarioSunshine,MarioGalaxy)
var sCurrentAssist		= false; 
var sIsForbiddenKey		= true;
var sCallbackUrl		= "";
var sCursorIdMain		= 0;
var sCursorIdOption		= 0;
var sCurrentMode		= FORCUS_MODE.MAIN
var sIsWaitRelease		= { B: false, X: false, Y: false }; //Si se presiona un boton
var sIsInClose			= false;
var sOptionTable		= {}

// Crea una tabla de opciones
var addOptionTableElement = function(key) {
	sOptionTable[key] = { query: key, button: "button_option_"+key, option_prefix: "option_"+key, is_reverse: false, is_enable: false };
}

// Variables de opciones de camara
var makeOptionTable = function() {
	addOptionTableElement( "camera_h" );
	addOptionTableElement( "camera_v" );
	addOptionTableElement( "pump_v" );
	addOptionTableElement( "fpcamera_h" );
	addOptionTableElement( "fpcamera_v" );
}
makeOptionTable(); //Crea tabla de opciones

// Abriendo
window.nx.addEventListener("message", function(event) {
	
	window.sound.play( "Open" ); //Reproduce Sonido de apertura
});

// Al finalizar la carga de sonido
var end_sound_load = function() {
	window.nx.sendMessage( "end_sound_load" ); //Detiene el reproductor de sonido
}

// Sonidos
window.sound.load( {
	"Open":		"../../audio/Open.wav",
	"Continue":	"../../audio/Continue.wav",
	"Move":		"../../audio/Move.wav",
	"Camera":	"../../audio/Camera.wav",
	"CameraIn":	"../../audio/CameraIn.wav",
	"CameraOut":"../../audio/CameraOut.wav",
	"Decide":	"../../audio/Decide.wav",
	"Exit":		"../../audio/Exit.wav",
	"Cancel":	"../../audio/Cancel.wav",
	"About":	"../../audio/About.wav",
}, end_sound_load );

//Obtiene parametros de URL
var loadUrlParam = function() {
	var param_hash = {};
	var param_str_list = location.search.substring(1).split( "&" );
	param_str_list.forEach( function(str) {
		var key_value = str.split( "=" );
		param_hash[key_value[0]] = key_value[1];
	} );

	//Recoge Titulo de Juego en Ejecucion
	sGameName = param_hash["game"]; //Videojuego abierto (64, Sunshine, Galaxy)
	document.getElementById("button_exit").style.display = "none";

	if( sGameName == "galaxy" ) //Si es MarioGalaxy
		sCurrentAssist = param_hash["current_assist"] == "on";
	else
		//Habilita el arreglo de 2 jugadores
		document.getElementById("button_guide_2p").style.display = "none"; 

	// Congifuracion
	//Nintendo 64
	if( sGameName == "64" ) {
		//Activa la opcion
		sOptionTable["camera_h"].is_enable = true;
		sOptionTable["fpcamera_h"].is_enable = true;
		sOptionTable["fpcamera_v"].is_enable = true;
	} 
	//Nintendo GameCube
	else if( sGameName == "sunshine" ) {
		sOptionTable["camera_h"].is_enable = true;
		sOptionTable["camera_v"].is_enable = true;
		sOptionTable["pump_v"].is_enable = true;
		sOptionTable["fpcamera_h"].is_enable = true;
		sOptionTable["fpcamera_v"].is_enable = true;
	} 
	//Nintendo Wii
	else if( sGameName == "galaxy" ) {
		sOptionTable["camera_h"].is_enable = true;
		sOptionTable["fpcamera_h"].is_enable = true;
		sOptionTable["fpcamera_v"].is_enable = true;
	}

	for( var key in sOptionTable ) {
		sOptionTable[key].is_reverse = param_hash[key] == "reverse";
	}
	syncOptionValue();
}

// Refresca las opciones cambiadas
var syncOptionValue = function() {
	// Borra todo el contenido al mismo tiempo
	for( var key in sOptionTable ) {
		document.getElementById(sOptionTable[key].button).style.display = "none";
		document.getElementById(sOptionTable[key].option_prefix + "_normal").style.display = "none";
		document.getElementById(sOptionTable[key].option_prefix + "_reverse").style.display = "none";
	}

	// Parametros para cada Juego
	for( var key in sOptionTable ) {
		if( !sOptionTable[key].is_enable ) continue;

		document.getElementById(sOptionTable[key].button).style.display = "flex";
		sOptionTable[key].is_enable = true;
		if( sOptionTable[key].is_reverse )	document.getElementById(sOptionTable[key].option_prefix + "_reverse").style.display = "inline";
		else								document.getElementById(sOptionTable[key].option_prefix + "_normal").style.display = "inline";
	}
}

// Funcion para consultar las configuraciones.
var makeOptionQuery = function() {
	var query = "";
	for( var key in sOptionTable ) {
		if( !sOptionTable[key].is_enable ) continue;
		if( query != "" ) query += "&";
		query += key;
		query += "=";
		query += (sOptionTable[key].is_reverse)? "reverse" : "normal";
	}

	return query;
}

// Restablece las configuraciones por defecto
var setOptionValueDefault = function() {
	if( sOptionTable["camera_h"].is_enable )	sOptionTable["camera_h"].is_reverse = false;
	if( sOptionTable["camera_v"].is_enable )	sOptionTable["camera_v"].is_reverse = false;
	if( sOptionTable["pump_v"].is_enable )		sOptionTable["pump_v"].is_reverse = false;
	if( sOptionTable["fpcamera_h"].is_enable )	sOptionTable["fpcamera_h"].is_reverse = false;
	if( sOptionTable["fpcamera_v"].is_enable )	sOptionTable["fpcamera_v"].is_reverse = false;
	syncOptionValue();
}

// Restablece las configuraciones en modo clasico
var setOptionValueClassic = function() {
	if( sOptionTable["camera_h"].is_enable )	sOptionTable["camera_h"].is_reverse = true;
	if( sOptionTable["camera_v"].is_enable )	sOptionTable["camera_v"].is_reverse = false;
	if( sOptionTable["pump_v"].is_enable )		sOptionTable["pump_v"].is_reverse = true;
	if( sOptionTable["fpcamera_h"].is_enable )	sOptionTable["fpcamera_h"].is_reverse = false;
	if( sOptionTable["fpcamera_v"].is_enable )	sOptionTable["fpcamera_v"].is_reverse = true;
	syncOptionValue();
}

// Deshabilitar la entrada de fotograma especificado (?
var forbidKey = function(msec) {
	sIsForbiddenKey = true;
	setTimeout( function() {
		if( !sIsInClose ) sIsForbiddenKey = false;
	}, msec );
}

// Mueve el cursor hacia adelante
var moveCursorNext = function(cursor_id, menu_table) {
	window.sound.play( "Move" );
	do {
		cursor_id = (cursor_id + 1) % menu_table.length;
	} while( document.getElementById(menu_table[cursor_id]).style.display == "none" );
	document.getElementById(menu_table[cursor_id]).focus();
	return cursor_id;
}

// Mueve el cursor hacia atrás
var moveCursorBack = function(cursor_id, menu_table) {
	window.sound.play( "Move" );
	do {
		cursor_id = (cursor_id - 1 + menu_table.length) % menu_table.length;
	} while( document.getElementById(menu_table[cursor_id]).style.display == "none" );
	document.getElementById(menu_table[cursor_id]).focus();
	return cursor_id;
}

// Cierra las opciones
var closeOption = function() {
	document.getElementById( "change_option" ).style.display = "none";
	sCurrentMode = FORCUS_MODE.MAIN
	document.getElementById("button_option").focus();
	window.sound.play( "CameraOut" );
	forbidKey( 200 );
}

// Confirmación para cerrar el juego
var cancelConfirm = function() {
	document.getElementById( "exit_game" ).style.display = "none";
	sCurrentMode = FORCUS_MODE.MAIN
	document.getElementById("button_exit").focus();
	window.sound.play( "Cancel" );
	forbidKey( 200 );
}

// Salir del Menu de Pausa
var exitHtml = function(callback_str, sound_label, is_need_option_query) {
	if( sIsForbiddenKey ) return;
	sIsForbiddenKey	= true;
	sIsInClose		= true;
	window.sound.play( sound_label );

	// Finaliza la animacion (.css)
	var div_all = document.getElementById( "all" );
	all.style.animation = "frame_out 0.2s ease-out 0s 1 normal forwards";
	setTimeout( function() {
		sCallbackUrl = "http://localhost/" + callback_str
		if( is_need_option_query ) sCallbackUrl += "?" + makeOptionQuery();
	}, 300 );
}

// Reanuda el juego
var exitForContinue = function() {
	exitHtml( "continue", "Continue", true );
}

// Cancelar
var exitForCancel = function() {
	exitHtml( "continue", "Cancel", true );
}

// Regresar al lanzador
var exitForReturnToLauncher = function() {
	exitHtml( "return_to_launcher", "Exit", false );
}

// Abrir el selector de jugadores (Wii)
var exitForControllerSupport = function() {
	if( sCurrentAssist )	exitHtml( "request_play_1p", "Decide", true );
	else					exitHtml( "request_play_2p", "Decide", true  );
}

// Procesa la actualizacion de JoyPad 
var update = function() {
	var pad;
	if( navigator.getGamepads && navigator.getGamepads()[0] ) {
		pad = navigator.getGamepads()[0];
		// － Boton Start
		if( pad.buttons[8].pressed && !sIsForbiddenKey ) {
			exitForCancel();
		}

		// ＋ Boton select
		if( pad.buttons[9].pressed && !sIsForbiddenKey ) {
			exitForCancel();
		}

		// Ｂ Boton B
		if( pad.buttons[0].pressed && !sIsForbiddenKey && !sIsWaitRelease.B ) {
			// Verifica si abrio una opcion.
			if( sCurrentMode == FORCUS_MODE.OPTION ) {
				closeOption( false );
			} else if( sCurrentMode == FORCUS_MODE.CONFIRM_EXIT ) {
				cancelConfirm();
			} else {
				exitForCancel();
			}
		}
		if( pad.buttons[0].pressed )	sIsWaitRelease.B = true;
		else							sIsWaitRelease.B = false;

		// Ｘ Boton X
		if( pad.buttons[3].pressed && !sIsForbiddenKey && !sIsWaitRelease.X ) {
			if( sCurrentMode == FORCUS_MODE.OPTION ) {
				setOptionValueDefault();
				window.sound.play( "Camera" );
			}
		}
		if( pad.buttons[3].pressed )	sIsWaitRelease.X = true;
		else							sIsWaitRelease.X = false;

		// Ｙ Boton Y
		if( pad.buttons[2].pressed && !sIsForbiddenKey && !sIsWaitRelease.Y ) {
			if( sCurrentMode == FORCUS_MODE.OPTION ) {
				setOptionValueClassic();
				window.sound.play( "Camera" );
			}
		}
		if( pad.buttons[2].pressed )	sIsWaitRelease.Y = true;
		else							sIsWaitRelease.Y = false;

		// Salir cuando se complete la URL final(?
		if( sCallbackUrl != "" ) {
			location.href = sCallbackUrl;
		}
	}
	requestAnimationFrame( update );
}

// Operación prohibida hasta el final de la animación de entrada de fotogramas(?
setTimeout( function() {
	sIsForbiddenKey = false;
}, 500 );

// Cursor
document.getElementById("all").addEventListener( "keydown", function(event) {
	if( sIsInClose ) return;

	// Confirmación para abandonar el juego.
	if( sCurrentMode == FORCUS_MODE.CONFIRM_EXIT ) {
		if( event.keyCode == KEY_CODE_LEFT ) {
			if( document.activeElement == document.getElementById("button_confirm_no") ) return;
			window.sound.play( "Move" );
			document.getElementById("button_confirm_no").focus();
		}

		if( event.keyCode == KEY_CODE_RIGHT ) {
			if( document.activeElement == document.getElementById("button_confirm_yes") ) return;
			window.sound.play( "Move" );
			document.getElementById("button_confirm_yes").focus();
		}
		return;
	}

	// Configuración
	if( sCurrentMode == FORCUS_MODE.OPTION ) {
		if( event.keyCode == KEY_CODE_DOWN ) {
			sCursorIdOption = moveCursorNext( sCursorIdOption, OPTION_MENU );
		}

		if( event.keyCode == KEY_CODE_UP ) {
			sCursorIdOption = moveCursorBack( sCursorIdOption, OPTION_MENU );
		}

		if( event.keyCode == KEY_CODE_LEFT || event.keyCode == KEY_CODE_RIGHT ) {
			if( sCursorIdOption == 0 )		sOptionTable["camera_h"].is_reverse = !sOptionTable["camera_h"].is_reverse;
			else if( sCursorIdOption == 1 )	sOptionTable["camera_v"].is_reverse = !sOptionTable["camera_v"].is_reverse;
			else if( sCursorIdOption == 2 )	sOptionTable["pump_v"].is_reverse = !sOptionTable["pump_v"].is_reverse;
			else if( sCursorIdOption == 3 )	sOptionTable["fpcamera_h"].is_reverse = !sOptionTable["fpcamera_h"].is_reverse;
			else if( sCursorIdOption == 4 )	sOptionTable["fpcamera_v"].is_reverse = !sOptionTable["fpcamera_v"].is_reverse;
			else							return;
			syncOptionValue();
			window.sound.play( "Move" );
		}
		return;
	}

	// Menú principal
	if( event.keyCode == KEY_CODE_DOWN ) {
		sCursorIdMain = moveCursorNext( sCursorIdMain, MAIN_MENU );
	}

	if( event.keyCode == KEY_CODE_UP ) {
		sCursorIdMain = moveCursorBack( sCursorIdMain, MAIN_MENU );
	}
} );

// Botón para continuar
var button_continue = document.getElementById( "button_continue" );
button_continue.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	exitForContinue();
} );

// Botón de guía de operación

var button_guide_basic = document.getElementById( "button_guide_basic" );
button_guide_basic.addEventListener( "focus", function(event) {
	var div_guide = document.getElementById( "guide_basic_" + sGameName );
	div_guide.style.display = "flex";
	forbidKey( 200 );
} );
button_guide_basic.addEventListener( "blur", function(event) {
	var div_guide = document.getElementById( "guide_basic_" + sGameName );
	div_guide.style.display = "none";
} );


// について...ここまで進んだら、私のファイルを変更しようと思うでしょう...したがって、これを使用してパックゲームを作成することは禁止されています。
var about = document.getElementById( "about" );
about.addEventListener( "focus", function(event) {
	var div_guide = document.getElementById( "guide_basic_about" );
	div_guide.style.display = "flex";
	forbidKey( 200 );
	window.sound.play( "About" );
} );
about.addEventListener( "blur", function(event) {
	var div_guide = document.getElementById( "guide_basic_about" );
	div_guide.style.display = "none";
} );

// Botón de guía de acción
var button_guide_advance = document.getElementById( "button_guide_advance" );
button_guide_advance.addEventListener( "focus", function(event) {
	var div_guide = document.getElementById( "guide_advance_" + sGameName );
	div_guide.style.display = "flex";
	forbidKey( 200 );
} );
button_guide_advance.addEventListener( "blur", function(event) {
	var div_guide = document.getElementById( "guide_advance_" + sGameName );
	div_guide.style.display = "none";
} );

// Botón para 2 Jugadores
var button_guide_2p = document.getElementById( "button_guide_2p" );
button_guide_2p.addEventListener( "focus", function(event) {
	document.getElementById( "guide_2p" ).style.display = "flex";
	if( sCurrentAssist )	document.getElementById( "guide_2p_play_end" ).style.display = "inline";
	else					document.getElementById( "guide_2p_play_start" ).style.display = "inline";
	forbidKey( 200 );
} );
button_guide_2p.addEventListener( "blur", function(event) {
	document.getElementById( "guide_2p" ).style.display = "none";
} );
button_guide_2p.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	exitForControllerSupport();
} );

// Botón de configuración
var button_option = document.getElementById( "button_option" );
button_option.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	sCurrentMode = FORCUS_MODE.OPTION;
	sCursorIdOption = 0;
	document.getElementById( "change_option" ).style.display = "block";
	if( sCurrentAssist ) {
		document.getElementById( "option_footer_1p" ).style.display = "none";
		document.getElementById( "option_footer_2p" ).style.display = "inline";
	} else {
		document.getElementById( "option_footer_1p" ).style.display = "inline";
		document.getElementById( "option_footer_2p" ).style.display = "none";
	}
	document.getElementById(OPTION_MENU[0]).focus();
	window.sound.play( "CameraIn" );
	forbidKey( 200 );
} );

// Si presiona A en el elemento de configuración, vaya al elemento siguiente y registre
var addEventListenerOptionNextByA = function() {
	for( var key in sOptionTable ) {
		var button = document.getElementById( sOptionTable[key].button );
		button.addEventListener( "keydown", function(event) {
			if( sIsForbiddenKey )				return;
			if( event.keyCode != KEY_CODE_A )	return;
			sCursorIdOption = moveCursorNext( sCursorIdOption, OPTION_MENU );
		} );
	}
}
addEventListenerOptionNextByA();

// Configuración [Atrás]
var button_option_ok = document.getElementById( "button_option_ok" );
button_option_ok.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	closeOption();
} );

// Botón para salir del juego
var button_exit = document.getElementById( "button_exit" );
button_exit.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	sCurrentMode = FORCUS_MODE.CONFIRM_EXIT;
	document.getElementById( "exit_game" ).style.display = "flex";
	document.getElementById("button_confirm_yes").focus();
	window.sound.play( "Continue" );
	forbidKey( 200 );
} );

// Confirmación para detener [Sí]
var button_confirm_yes = document.getElementById( "button_confirm_yes" );
button_confirm_yes.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	exitForReturnToLauncher();
} );

// Confirmación para detener [No]
var button_confirm_no = document.getElementById( "button_confirm_no" );
button_confirm_no.addEventListener( "keydown", function(event) {
	if( sIsForbiddenKey )				return;
	if( event.keyCode != KEY_CODE_A )	return;
	cancelConfirm();
} );

// varias configuraciones
window.nx.setCursorScrollSpeed( 0 );
window.nx.footer.unsetAssign( "B" );
window.nx.footer.unsetAssign( "X" );
document.getElementById(MAIN_MENU[0]).focus();

// Análisis de parámetros
loadUrlParam();
update();


